from django.core.management import BaseCommand, CommandError
from flatblocks.models import FlatBlock


class Command(BaseCommand):
    help = "Delete a flatblock with the given slug"

    def handle(self, *args, **options):
        if len(args) != 1:
            raise CommandError("This command requires the slug of the "
                               "flatblock as its first argument")
        slug = args[0]
        try:
            FlatBlock.objects.get(slug=slug).delete()
        except FlatBlock.DoesNotExist:
            raise CommandError("The requested flatblock doesn't exist")



from django.contrib import admin
from flatblocks.models import FlatBlock


class FlatBlockAdmin(admin.ModelAdmin):
    ordering = ['slug', ]
    list_display = ('slug', 'header')
    search_fields = ('slug', 'header', 'content')

admin.site.register(FlatBlock, FlatBlockAdmin)



from django.core.management import BaseCommand, CommandError
from django.db import IntegrityError
from flatblocks.models import FlatBlock


class Command(BaseCommand):
    help = "Create a new flatblock with the given slug"

    def handle(self, *args, **options):
        if len(args) != 1:
            raise CommandError("This command requires the slug of the new "
                               "flatblock as its first argument")
        slug = args[0]
        block = FlatBlock(header="[{0}]".format(slug),
                          content="Empty flatblock",
                          slug=slug)
        try:
            block.save()
        except IntegrityError:
            raise CommandError("A flatblock with this name already exists")



"""
This module offers one templatetag called "flatblock" which allows you to
easily embed small text-snippets (like for example the help section of a page)
into a template.

It requires one argument, and accepts several extra keywords:

    {% flatblock {slug} [evaluated={bool}] [using={template}] %}

    slug::
        The slug/key of the text (for example 'contact_help').

    evaluated::
        If set to True, the content and header of the FlatBlock will be
        rendered as a Template before being passed to the template.

        This allows you to embed template tags into your FlatBlocks.

        The original values for the content and header will be saved to
        raw_content and raw_header respectively.

    using::
        The template to render the FlatBlock with.  If not supplied, will
        default to "flatblocks/flatblock.html".

        If set to False, will not be used, and the output of the tag will be
        the ``content`` property of the FlatBlock.

Example::

    {% load flatblock_tags %}

    ...

    {% flatblock 'contact_help' %}
    {% flatblock name_in_variable %}

The 'flatblock' template tag acts like an inclusiontag and operates on the
``flatblock/flatblock.html`` template file, which gets (besides the global
context) also the ``flatblock`` variable passed.

Compared to the original implementation this includes not only the block's
content but the whole object inclusing title, slug and id. This way you
can easily for example offer administrative operations (like editing)
within that template.

"""
from __future__ import absolute_import

import logging

from django import VERSION, template
from django.template.loader import render_to_string
from flatblocks import settings

if VERSION >= (1, 7):
    from django.apps import apps
    get_model = apps.get_model
else:
    from django.db import models
    get_model = models.get_model


register = template.Library()
logger = logging.getLogger(__name__)

FlatBlock = get_model('flatblocks', 'flatblock')


@register.simple_tag(takes_context=True)
def flatblock(context, slug, evaluated=False,
              using='flatblocks/flatblock.html'):

    if not settings.AUTOCREATE_STATIC_BLOCKS:
        try:
            flatblock = FlatBlock.objects.get(slug=slug)
        except FlatBlock.DoesNotExist:
            return ''
    else:
        flatblock, _ = FlatBlock.objects.get_or_create(slug=slug,
            defaults={'content': slug}
        )

    if evaluated:
        flatblock.raw_content = flatblock.content
        flatblock.raw_header = flatblock.header
        flatblock.content = template.Template(flatblock.content).render(context)
        flatblock.header = template.Template(flatblock.header).render(context)

    if using:
        ctx = context.flatten()
        ctx['flatblock'] = flatblock
        result = render_to_string(using, ctx)
    else:
        result = flatblock.content

    return result


@register.simple_tag(takes_context=True)
def plain_flatblock(context, slug, evaluated=False):
    return flatblock(context, slug, evaluated=evaluated, using=None)



from django.http import HttpResponse, HttpResponseForbidden
from django.shortcuts import get_object_or_404, redirect, render
from django.utils.translation import gettext as _
from flatblocks.forms import FlatBlockForm
from flatblocks.models import FlatBlock


def edit(request, pk, modelform_class=FlatBlockForm, permission_check=None,
         template_name='flatblocks/edit.html', success_url=None):
    """
    This view provides a simple editor implementation for flatblocks.

    There are two customization hooks. First of all you can specify your own
    ModelForm class by passing it to the view using the ``modelform_class``
    keyword-argument.

    The other entry point helps you check permissions: Pass a simple function
    via the ``permission_check`` keyword-argument in order to check
    permissions on the flatblock-level::

        def my_perm_check(request, flatblock):
            return request.user.is_staff

        # ...

        urlpatterns('flatblocks.views',
            url('flatblocks/(?P<pk>\d+)/edit/$', 'edit',
                kwargs={'permission_check': my_perm_check}),
        )

    The contract here is pretty simple: If the function returns False, the
    view will return HttpResponseForbidden. Otherwise it will pass.  So if you
    want to do some fancy redirects if the permissions are wrong, return your
    own HttpResponse-object/-subclass.

    If everything is alright with the permissions, simply return True.
    """
    flatblock = get_object_or_404(FlatBlock, pk=pk)
    if permission_check:
        permcheck_result = permission_check(request, flatblock)
        if permcheck_result is False:
            return HttpResponseForbidden(
                _('You are not allowed to edit this flatblock'))
        if isinstance(permcheck_result, HttpResponse):
            return permcheck_result

    session_key = 'flatblock.origin.%d' % (int(pk), )
    if request.method == 'POST':
        origin = request.session.get(session_key,
                                     request.META.get('HTTP_REFERER', '/'))
        form = modelform_class(request.POST, instance=flatblock)
        if form.is_valid():
            instance = form.save(commit=False)
            instance.slug = flatblock.slug
            instance.save()
            del request.session[session_key]
            redirect_to = success_url if success_url else origin
            return redirect(redirect_to)
    else:
        origin = request.META.get('HTTP_REFERER', '/')
        # Don't set origin to this view's url no matter what
        origin = (
            request.session.get(session_key, '/')
            if origin == request.get_full_path()
            else origin
        )
        form = modelform_class(instance=flatblock)
        request.session[session_key] = origin
    return render(request, template_name, {
        'form': form,
        'origin': origin,
        'flatblock': flatblock,
    })



import os

from django.conf import settings
from django.core.management.base import BaseCommand
from django.template.loader import get_template
from flatblocks.models import FlatBlock
from flatblocks.templatetags.flatblock_tags import FlatBlockNode


class Command(BaseCommand):
    help = "List unassigned flatblocks in the templates"

    def handle(self, *args, **options):
        save_nodes = (len(args) and args[0] == 'create')
        flatblock_nodes = set()
        print_nodes = []

        # get list of templates
        for templ_dir in settings.TEMPLATE_DIRS:
            for path, dirlist, fnlist in os.walk(templ_dir):
                for fn in fnlist:
                    try:
                        t = get_template(os.path.join(path, fn))
                        flatblock_nodes.update(
                            flatblock_nodes.update(
                                node.slug
                                for node in t.nodelist.get_nodes_by_type(FlatBlockNode)
                            )
                        )
                    except:
                        # Should log at debug level?
                        pass

        # check if flatblocks have entry in database
        for node in flatblock_nodes:
            if not FlatBlock.objects.filter(slug=node).exists():
                # if create argument was supplied, save empty nodes
                if save_nodes:
                    FlatBlock.objects.create(header="[{0}]".format(node),
                                             content="Generated flatblock",
                                             slug=node)
                print_nodes.append(node)

        if print_nodes:
            if save_nodes:
                self.stdout.write("Following nodes were created:")
            self.stdout.write("\n".join(print_nodes))
        else:
            self.stdout.write("All FlatBlock items are in database")




from django.db import models
from django.utils.translation import gettext_lazy as _


class FlatBlock(models.Model):
    """
    Think of a flatblock as a flatpage but for just part of a site. It's
    basically a piece of content with a given name (slug) and an optional
    title (header) which you can, for example, use in a sidebar of a website.
    """
    slug = models.CharField(_('Slug'), max_length=255, unique=True,
        help_text=_("A unique name used for reference in the templates")
    )
    header = models.CharField(_('Header'), blank=True, max_length=255,
        help_text=_("An optional header for this content")
    )
    content = models.TextField(verbose_name=_('Content'), blank=True)

    # Helper attributes used if content should be evaluated in order to
    # represent the original content.
    raw_content = None
    raw_header = None

    def __str__(self):
        return self.slug

    class Meta:
        verbose_name = _('Flat block')
        verbose_name_plural = _('Flat blocks')



from django.contrib.admin.views.decorators import staff_member_required
from django.urls import re_path
from flatblocks.views import edit

urlpatterns = [
    re_path(
        r"^edit/(?P<pk>\d+)/$",
        staff_member_required(edit),
        name="flatblocks-edit",
    ),
]



# encoding: utf-8
import datetime

from django.db import models
from south.db import db
from south.v2 import SchemaMigration


class Migration(SchemaMigration):

    def forwards(self, orm):

        # Adding model 'FlatBlock'
        db.create_table('flatblocks_flatblock', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('slug', self.gf('django.db.models.fields.CharField')(unique=True, max_length=255)),
            ('header', self.gf('django.db.models.fields.CharField')(max_length=255, null=True, blank=True)),
            ('content', self.gf('django.db.models.fields.TextField')(null=True, blank=True)),
        ))
        db.send_create_signal('flatblocks', ['FlatBlock'])

    def backwards(self, orm):

        # Deleting model 'FlatBlock'
        db.delete_table('flatblocks_flatblock')

    models = {
        'flatblocks.flatblock': {
            'Meta': {'object_name': 'FlatBlock'},
            'content': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'header': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'slug': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '255'})
        }
    }

    complete_apps = ['flatblocks']



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='FlatBlock',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('slug', models.CharField(help_text='A unique name used for reference in the templates', unique=True, max_length=255, verbose_name='Slug')),
                ('header', models.CharField(help_text='An optional header for this content', max_length=255, verbose_name='Header', blank=True)),
                ('content', models.TextField(verbose_name='Content', blank=True)),
            ],
            options={
                'verbose_name': 'Flat block',
                'verbose_name_plural': 'Flat blocks',
            },
            bases=(models.Model,),
        ),
    ]



import io

from setuptools import setup, find_packages


setup(
    name='django-flatblocks',
    version='1.0.0',
    description='django-flatblocks acts like django.contrib.flatpages but '
                'for parts of a page; like an editable help box you want '
                'show alongside the main content.',
    long_description=io.open('README.rst', encoding='utf-8').read(),
    keywords='django apps',
    license='New BSD License',
    author='Horst Gutmann, Curtis Maloney',
    author_email='curtis@tinbrain.net',
    url='http://github.com/funkybob/django-flatblocks/',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Plugins',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 2.2',
        'Framework :: Django :: 3.0',
        'Framework :: Django :: 3.1',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: Implementation :: PyPy',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
    packages=find_packages(exclude=['tests']),
    package_data={
        'flatblocks': [
            'templates/flatblocks/*.html',
            'locale/*/*/*.mo',
            'locale/*/*/*.po',
        ]
    },
    zip_safe=False,
    requires = [
        'Django (>=2.2)',
    ],
)



# -*- coding: utf-8 -*-
import datetime

from django.db import models
from south.db import db
from south.v2 import SchemaMigration


class Migration(SchemaMigration):

    def forwards(self, orm):

        # Changing field 'FlatBlock.content'
        db.alter_column(u'flatblocks_flatblock', 'content', self.gf('django.db.models.fields.TextField')(default=''))

        # Changing field 'FlatBlock.header'
        db.alter_column(u'flatblocks_flatblock', 'header', self.gf('django.db.models.fields.CharField')(default='', max_length=255))

    def backwards(self, orm):

        # Changing field 'FlatBlock.content'
        db.alter_column(u'flatblocks_flatblock', 'content', self.gf('django.db.models.fields.TextField')(null=True))

        # Changing field 'FlatBlock.header'
        db.alter_column(u'flatblocks_flatblock', 'header', self.gf('django.db.models.fields.CharField')(max_length=255, null=True))

    models = {
        u'flatblocks.flatblock': {
            'Meta': {'object_name': 'FlatBlock'},
            'content': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'header': ('django.db.models.fields.CharField', [], {'max_length': '255', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'slug': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '255'})
        }
    }

    complete_apps = ['flatblocks']
